package com.tinkerforge;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickDCActions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Set Velocity")
    public void brickDCSetVelocity(
            @ActionInput(name = "velocity") short velocity) throws TinkerforgeException {
        ((BrickDC)this.handler.getDevice()).setVelocity(velocity);
    }

    public static void brickDCSetVelocity(@Nullable ThingActions actions, short velocity) throws TinkerforgeException {
        if (actions instanceof BrickDCActions) {
            ((BrickDCActions) actions).brickDCSetVelocity(velocity);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickDCActions class.");
        }
    }

    @RuleAction(label = "Get Velocity")
    public @ActionOutput(name = "velocity", type="short")
           Map<String, Object> brickDCGetVelocity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        short value = ((BrickDC)this.handler.getDevice()).getVelocity();
        result.put("velocity", value);
        return result;
    }

    public static Map<String, Object> brickDCGetVelocity(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickDCActions) {
            return ((BrickDCActions) actions).brickDCGetVelocity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickDCActions class.");
        }
    }

    @RuleAction(label = "Get Current Velocity")
    public @ActionOutput(name = "velocity", type="short")
           Map<String, Object> brickDCGetCurrentVelocity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        short value = ((BrickDC)this.handler.getDevice()).getCurrentVelocity();
        result.put("velocity", value);
        return result;
    }

    public static Map<String, Object> brickDCGetCurrentVelocity(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickDCActions) {
            return ((BrickDCActions) actions).brickDCGetCurrentVelocity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickDCActions class.");
        }
    }

    @RuleAction(label = "Set Acceleration")
    public void brickDCSetAcceleration(
            @ActionInput(name = "acceleration") int acceleration) throws TinkerforgeException {
        ((BrickDC)this.handler.getDevice()).setAcceleration(acceleration);
    }

    public static void brickDCSetAcceleration(@Nullable ThingActions actions, int acceleration) throws TinkerforgeException {
        if (actions instanceof BrickDCActions) {
            ((BrickDCActions) actions).brickDCSetAcceleration(acceleration);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickDCActions class.");
        }
    }

    @RuleAction(label = "Get Acceleration")
    public @ActionOutput(name = "acceleration", type="int")
           Map<String, Object> brickDCGetAcceleration(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickDC)this.handler.getDevice()).getAcceleration();
        result.put("acceleration", value);
        return result;
    }

    public static Map<String, Object> brickDCGetAcceleration(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickDCActions) {
            return ((BrickDCActions) actions).brickDCGetAcceleration();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickDCActions class.");
        }
    }

    @RuleAction(label = "Full Brake")
    public void brickDCFullBrake(
            ) throws TinkerforgeException {
        ((BrickDC)this.handler.getDevice()).fullBrake();
    }

    public static void brickDCFullBrake(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickDCActions) {
            ((BrickDCActions) actions).brickDCFullBrake();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickDCActions class.");
        }
    }

    @RuleAction(label = "Enable")
    public void brickDCEnable(
            ) throws TinkerforgeException {
        ((BrickDC)this.handler.getDevice()).enable();
    }

    public static void brickDCEnable(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickDCActions) {
            ((BrickDCActions) actions).brickDCEnable();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickDCActions class.");
        }
    }

    @RuleAction(label = "Disable")
    public void brickDCDisable(
            ) throws TinkerforgeException {
        ((BrickDC)this.handler.getDevice()).disable();
    }

    public static void brickDCDisable(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickDCActions) {
            ((BrickDCActions) actions).brickDCDisable();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickDCActions class.");
        }
    }

    @RuleAction(label = "Is Enabled")
    public @ActionOutput(name = "enabled", type="boolean")
           Map<String, Object> brickDCIsEnabled(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        boolean value = ((BrickDC)this.handler.getDevice()).isEnabled();
        result.put("enabled", value);
        return result;
    }

    public static Map<String, Object> brickDCIsEnabled(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickDCActions) {
            return ((BrickDCActions) actions).brickDCIsEnabled();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickDCActions class.");
        }
    }

    @RuleAction(label = "Set PWM Frequency")
    public void brickDCSetPWMFrequency(
            @ActionInput(name = "frequency") int frequency) throws TinkerforgeException {
        ((BrickDC)this.handler.getDevice()).setPWMFrequency(frequency);
    }

    public static void brickDCSetPWMFrequency(@Nullable ThingActions actions, int frequency) throws TinkerforgeException {
        if (actions instanceof BrickDCActions) {
            ((BrickDCActions) actions).brickDCSetPWMFrequency(frequency);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickDCActions class.");
        }
    }

    @RuleAction(label = "Get PWM Frequency")
    public @ActionOutput(name = "frequency", type="int")
           Map<String, Object> brickDCGetPWMFrequency(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickDC)this.handler.getDevice()).getPWMFrequency();
        result.put("frequency", value);
        return result;
    }

    public static Map<String, Object> brickDCGetPWMFrequency(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickDCActions) {
            return ((BrickDCActions) actions).brickDCGetPWMFrequency();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickDCActions class.");
        }
    }

    @RuleAction(label = "Get Stack Input Voltage")
    public @ActionOutput(name = "voltage", type="int")
           Map<String, Object> brickDCGetStackInputVoltage(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickDC)this.handler.getDevice()).getStackInputVoltage();
        result.put("voltage", value);
        return result;
    }

    public static Map<String, Object> brickDCGetStackInputVoltage(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickDCActions) {
            return ((BrickDCActions) actions).brickDCGetStackInputVoltage();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickDCActions class.");
        }
    }

    @RuleAction(label = "Get External Input Voltage")
    public @ActionOutput(name = "voltage", type="int")
           Map<String, Object> brickDCGetExternalInputVoltage(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickDC)this.handler.getDevice()).getExternalInputVoltage();
        result.put("voltage", value);
        return result;
    }

    public static Map<String, Object> brickDCGetExternalInputVoltage(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickDCActions) {
            return ((BrickDCActions) actions).brickDCGetExternalInputVoltage();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickDCActions class.");
        }
    }

    @RuleAction(label = "Get Current Consumption")
    public @ActionOutput(name = "voltage", type="int")
           Map<String, Object> brickDCGetCurrentConsumption(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickDC)this.handler.getDevice()).getCurrentConsumption();
        result.put("voltage", value);
        return result;
    }

    public static Map<String, Object> brickDCGetCurrentConsumption(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickDCActions) {
            return ((BrickDCActions) actions).brickDCGetCurrentConsumption();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickDCActions class.");
        }
    }

    @RuleAction(label = "Set Drive Mode")
    public void brickDCSetDriveMode(
            @ActionInput(name = "mode") short mode) throws TinkerforgeException {
        ((BrickDC)this.handler.getDevice()).setDriveMode(mode);
    }

    public static void brickDCSetDriveMode(@Nullable ThingActions actions, short mode) throws TinkerforgeException {
        if (actions instanceof BrickDCActions) {
            ((BrickDCActions) actions).brickDCSetDriveMode(mode);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickDCActions class.");
        }
    }

    @RuleAction(label = "Get Drive Mode")
    public @ActionOutput(name = "mode", type="short")
           Map<String, Object> brickDCGetDriveMode(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        short value = ((BrickDC)this.handler.getDevice()).getDriveMode();
        result.put("mode", value);
        return result;
    }

    public static Map<String, Object> brickDCGetDriveMode(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickDCActions) {
            return ((BrickDCActions) actions).brickDCGetDriveMode();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickDCActions class.");
        }
    }

    @RuleAction(label = "Get Minimum Voltage")
    public @ActionOutput(name = "voltage", type="int")
           Map<String, Object> brickDCGetMinimumVoltage(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickDC)this.handler.getDevice()).getMinimumVoltage();
        result.put("voltage", value);
        return result;
    }

    public static Map<String, Object> brickDCGetMinimumVoltage(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickDCActions) {
            return ((BrickDCActions) actions).brickDCGetMinimumVoltage();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickDCActions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="short[]")
           @ActionOutput(name = "firmwareVersion", type="short[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickDCGetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickDC.Identity value = ((BrickDC)this.handler.getDevice()).getIdentity();
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickDCGetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickDCActions) {
            return ((BrickDCActions) actions).brickDCGetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickDCActions class.");
        }
    }

    @RuleAction(label = "Is Status LED Enabled")
    public @ActionOutput(name = "enabled", type="boolean")
           Map<String, Object> brickDCIsStatusLEDEnabled(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        boolean value = ((BrickDC)this.handler.getDevice()).isStatusLEDEnabled();
        result.put("enabled", value);
        return result;
    }

    public static Map<String, Object> brickDCIsStatusLEDEnabled(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickDCActions) {
            return ((BrickDCActions) actions).brickDCIsStatusLEDEnabled();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickDCActions class.");
        }
    }

    @RuleAction(label = "Get Chip Temperature")
    public @ActionOutput(name = "temperature", type="short")
           Map<String, Object> brickDCGetChipTemperature(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        short value = ((BrickDC)this.handler.getDevice()).getChipTemperature();
        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickDCGetChipTemperature(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickDCActions) {
            return ((BrickDCActions) actions).brickDCGetChipTemperature();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickDCActions class.");
        }
    }

    @RuleAction(label = "Get Send Timeout Count")
    public @ActionOutput(name = "timeoutCount", type="long")
           Map<String, Object> brickDCGetSendTimeoutCount(
            @ActionInput(name = "communicationMethod") short communicationMethod) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        long value = ((BrickDC)this.handler.getDevice()).getSendTimeoutCount(communicationMethod);
        result.put("timeoutCount", value);
        return result;
    }

    public static Map<String, Object> brickDCGetSendTimeoutCount(@Nullable ThingActions actions, short communicationMethod) throws TinkerforgeException {
        if (actions instanceof BrickDCActions) {
            return ((BrickDCActions) actions).brickDCGetSendTimeoutCount(communicationMethod);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickDCActions class.");
        }
    }

    @RuleAction(label = "Get SPITFP Baudrate Config")
    public @ActionOutput(name = "enableDynamicBaudrate", type="boolean")
           @ActionOutput(name = "minimumDynamicBaudrate", type="long")
           Map<String, Object> brickDCGetSPITFPBaudrateConfig(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickDC.SPITFPBaudrateConfig value = ((BrickDC)this.handler.getDevice()).getSPITFPBaudrateConfig();
        result.put("enableDynamicBaudrate", value.enableDynamicBaudrate);
        result.put("minimumDynamicBaudrate", value.minimumDynamicBaudrate);
        return result;
    }

    public static Map<String, Object> brickDCGetSPITFPBaudrateConfig(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickDCActions) {
            return ((BrickDCActions) actions).brickDCGetSPITFPBaudrateConfig();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickDCActions class.");
        }
    }

    @RuleAction(label = "Get SPITFP Error Count")
    public @ActionOutput(name = "errorCountACKChecksum", type="long")
           @ActionOutput(name = "errorCountMessageChecksum", type="long")
           @ActionOutput(name = "errorCountFrame", type="long")
           @ActionOutput(name = "errorCountOverflow", type="long")
           Map<String, Object> brickDCGetSPITFPErrorCount(
            @ActionInput(name = "brickletPort") char brickletPort) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickDC.SPITFPErrorCount value = ((BrickDC)this.handler.getDevice()).getSPITFPErrorCount(brickletPort);
        result.put("errorCountACKChecksum", value.errorCountACKChecksum);
        result.put("errorCountMessageChecksum", value.errorCountMessageChecksum);
        result.put("errorCountFrame", value.errorCountFrame);
        result.put("errorCountOverflow", value.errorCountOverflow);
        return result;
    }

    public static Map<String, Object> brickDCGetSPITFPErrorCount(@Nullable ThingActions actions, char brickletPort) throws TinkerforgeException {
        if (actions instanceof BrickDCActions) {
            return ((BrickDCActions) actions).brickDCGetSPITFPErrorCount(brickletPort);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickDCActions class.");
        }
    }

    @RuleAction(label = "Get SPITFP Baudrate")
    public @ActionOutput(name = "baudrate", type="long")
           Map<String, Object> brickDCGetSPITFPBaudrate(
            @ActionInput(name = "brickletPort") char brickletPort) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        long value = ((BrickDC)this.handler.getDevice()).getSPITFPBaudrate(brickletPort);
        result.put("baudrate", value);
        return result;
    }

    public static Map<String, Object> brickDCGetSPITFPBaudrate(@Nullable ThingActions actions, char brickletPort) throws TinkerforgeException {
        if (actions instanceof BrickDCActions) {
            return ((BrickDCActions) actions).brickDCGetSPITFPBaudrate(brickletPort);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickDCActions class.");
        }
    }
}
