package com.tinkerforge;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletAccelerometerActions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Get Acceleration")
    public @ActionOutput(name = "x", type="short")
           @ActionOutput(name = "y", type="short")
           @ActionOutput(name = "z", type="short")
           Map<String, Object> brickletAccelerometerGetAcceleration(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletAccelerometer.Acceleration value = ((BrickletAccelerometer)this.handler.getDevice()).getAcceleration();
        result.put("x", value.x);
        result.put("y", value.y);
        result.put("z", value.z);
        return result;
    }

    public static Map<String, Object> brickletAccelerometerGetAcceleration(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletAccelerometerActions) {
            return ((BrickletAccelerometerActions) actions).brickletAccelerometerGetAcceleration();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletAccelerometerActions class.");
        }
    }

    @RuleAction(label = "Get Temperature")
    public @ActionOutput(name = "temperature", type="short")
           Map<String, Object> brickletAccelerometerGetTemperature(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        short value = ((BrickletAccelerometer)this.handler.getDevice()).getTemperature();
        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickletAccelerometerGetTemperature(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletAccelerometerActions) {
            return ((BrickletAccelerometerActions) actions).brickletAccelerometerGetTemperature();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletAccelerometerActions class.");
        }
    }

    @RuleAction(label = "Is LED On")
    public @ActionOutput(name = "on", type="boolean")
           Map<String, Object> brickletAccelerometerIsLEDOn(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        boolean value = ((BrickletAccelerometer)this.handler.getDevice()).isLEDOn();
        result.put("on", value);
        return result;
    }

    public static Map<String, Object> brickletAccelerometerIsLEDOn(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletAccelerometerActions) {
            return ((BrickletAccelerometerActions) actions).brickletAccelerometerIsLEDOn();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletAccelerometerActions class.");
        }
    }

    @RuleAction(label = "Get Configuration")
    public @ActionOutput(name = "dataRate", type="short")
           @ActionOutput(name = "fullScale", type="short")
           @ActionOutput(name = "filterBandwidth", type="short")
           Map<String, Object> brickletAccelerometerGetConfiguration(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletAccelerometer.Configuration value = ((BrickletAccelerometer)this.handler.getDevice()).getConfiguration();
        result.put("dataRate", value.dataRate);
        result.put("fullScale", value.fullScale);
        result.put("filterBandwidth", value.filterBandwidth);
        return result;
    }

    public static Map<String, Object> brickletAccelerometerGetConfiguration(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletAccelerometerActions) {
            return ((BrickletAccelerometerActions) actions).brickletAccelerometerGetConfiguration();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletAccelerometerActions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="short[]")
           @ActionOutput(name = "firmwareVersion", type="short[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickletAccelerometerGetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletAccelerometer.Identity value = ((BrickletAccelerometer)this.handler.getDevice()).getIdentity();
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletAccelerometerGetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletAccelerometerActions) {
            return ((BrickletAccelerometerActions) actions).brickletAccelerometerGetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletAccelerometerActions class.");
        }
    }
}
