package com.tinkerforge;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletAnalogInV3Actions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Get Voltage")
    public @ActionOutput(name = "voltage", type="int")
           Map<String, Object> brickletAnalogInV3GetVoltage(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletAnalogInV3)this.handler.getDevice()).getVoltage();
        result.put("voltage", value);
        return result;
    }

    public static Map<String, Object> brickletAnalogInV3GetVoltage(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletAnalogInV3Actions) {
            return ((BrickletAnalogInV3Actions) actions).brickletAnalogInV3GetVoltage();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletAnalogInV3Actions class.");
        }
    }

    @RuleAction(label = "Get Oversampling")
    public @ActionOutput(name = "oversampling", type="int")
           Map<String, Object> brickletAnalogInV3GetOversampling(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletAnalogInV3)this.handler.getDevice()).getOversampling();
        result.put("oversampling", value);
        return result;
    }

    public static Map<String, Object> brickletAnalogInV3GetOversampling(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletAnalogInV3Actions) {
            return ((BrickletAnalogInV3Actions) actions).brickletAnalogInV3GetOversampling();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletAnalogInV3Actions class.");
        }
    }

    @RuleAction(label = "Get Calibration")
    public @ActionOutput(name = "offset", type="int")
           @ActionOutput(name = "multiplier", type="int")
           @ActionOutput(name = "divisor", type="int")
           Map<String, Object> brickletAnalogInV3GetCalibration(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletAnalogInV3.Calibration value = ((BrickletAnalogInV3)this.handler.getDevice()).getCalibration();
        result.put("offset", value.offset);
        result.put("multiplier", value.multiplier);
        result.put("divisor", value.divisor);
        return result;
    }

    public static Map<String, Object> brickletAnalogInV3GetCalibration(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletAnalogInV3Actions) {
            return ((BrickletAnalogInV3Actions) actions).brickletAnalogInV3GetCalibration();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletAnalogInV3Actions class.");
        }
    }

    @RuleAction(label = "Read UID")
    public @ActionOutput(name = "uid", type="long")
           Map<String, Object> brickletAnalogInV3ReadUID(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        long value = ((BrickletAnalogInV3)this.handler.getDevice()).readUID();
        result.put("uid", value);
        return result;
    }

    public static Map<String, Object> brickletAnalogInV3ReadUID(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletAnalogInV3Actions) {
            return ((BrickletAnalogInV3Actions) actions).brickletAnalogInV3ReadUID();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletAnalogInV3Actions class.");
        }
    }

    @RuleAction(label = "Get Chip Temperature")
    public @ActionOutput(name = "temperature", type="int")
           Map<String, Object> brickletAnalogInV3GetChipTemperature(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletAnalogInV3)this.handler.getDevice()).getChipTemperature();
        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickletAnalogInV3GetChipTemperature(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletAnalogInV3Actions) {
            return ((BrickletAnalogInV3Actions) actions).brickletAnalogInV3GetChipTemperature();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletAnalogInV3Actions class.");
        }
    }

    @RuleAction(label = "Get Status LED Config")
    public @ActionOutput(name = "config", type="int")
           Map<String, Object> brickletAnalogInV3GetStatusLEDConfig(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletAnalogInV3)this.handler.getDevice()).getStatusLEDConfig();
        result.put("config", value);
        return result;
    }

    public static Map<String, Object> brickletAnalogInV3GetStatusLEDConfig(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletAnalogInV3Actions) {
            return ((BrickletAnalogInV3Actions) actions).brickletAnalogInV3GetStatusLEDConfig();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletAnalogInV3Actions class.");
        }
    }

    @RuleAction(label = "Get Bootloader Mode")
    public @ActionOutput(name = "mode", type="int")
           Map<String, Object> brickletAnalogInV3GetBootloaderMode(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletAnalogInV3)this.handler.getDevice()).getBootloaderMode();
        result.put("mode", value);
        return result;
    }

    public static Map<String, Object> brickletAnalogInV3GetBootloaderMode(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletAnalogInV3Actions) {
            return ((BrickletAnalogInV3Actions) actions).brickletAnalogInV3GetBootloaderMode();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletAnalogInV3Actions class.");
        }
    }

    @RuleAction(label = "Get SPITFP Error Count")
    public @ActionOutput(name = "errorCountAckChecksum", type="long")
           @ActionOutput(name = "errorCountMessageChecksum", type="long")
           @ActionOutput(name = "errorCountFrame", type="long")
           @ActionOutput(name = "errorCountOverflow", type="long")
           Map<String, Object> brickletAnalogInV3GetSPITFPErrorCount(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletAnalogInV3.SPITFPErrorCount value = ((BrickletAnalogInV3)this.handler.getDevice()).getSPITFPErrorCount();
        result.put("errorCountAckChecksum", value.errorCountAckChecksum);
        result.put("errorCountMessageChecksum", value.errorCountMessageChecksum);
        result.put("errorCountFrame", value.errorCountFrame);
        result.put("errorCountOverflow", value.errorCountOverflow);
        return result;
    }

    public static Map<String, Object> brickletAnalogInV3GetSPITFPErrorCount(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletAnalogInV3Actions) {
            return ((BrickletAnalogInV3Actions) actions).brickletAnalogInV3GetSPITFPErrorCount();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletAnalogInV3Actions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="int[]")
           @ActionOutput(name = "firmwareVersion", type="int[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickletAnalogInV3GetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletAnalogInV3.Identity value = ((BrickletAnalogInV3)this.handler.getDevice()).getIdentity();
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletAnalogInV3GetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletAnalogInV3Actions) {
            return ((BrickletAnalogInV3Actions) actions).brickletAnalogInV3GetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletAnalogInV3Actions class.");
        }
    }
}
