package com.tinkerforge;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletAnalogOutActions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Get Voltage")
    public @ActionOutput(name = "voltage", type="int")
           Map<String, Object> brickletAnalogOutGetVoltage(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletAnalogOut)this.handler.getDevice()).getVoltage();
        result.put("voltage", value);
        return result;
    }

    public static Map<String, Object> brickletAnalogOutGetVoltage(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletAnalogOutActions) {
            return ((BrickletAnalogOutActions) actions).brickletAnalogOutGetVoltage();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletAnalogOutActions class.");
        }
    }

    @RuleAction(label = "Get Mode")
    public @ActionOutput(name = "mode", type="short")
           Map<String, Object> brickletAnalogOutGetMode(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        short value = ((BrickletAnalogOut)this.handler.getDevice()).getMode();
        result.put("mode", value);
        return result;
    }

    public static Map<String, Object> brickletAnalogOutGetMode(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletAnalogOutActions) {
            return ((BrickletAnalogOutActions) actions).brickletAnalogOutGetMode();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletAnalogOutActions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="short[]")
           @ActionOutput(name = "firmwareVersion", type="short[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickletAnalogOutGetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletAnalogOut.Identity value = ((BrickletAnalogOut)this.handler.getDevice()).getIdentity();
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletAnalogOutGetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletAnalogOutActions) {
            return ((BrickletAnalogOutActions) actions).brickletAnalogOutGetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletAnalogOutActions class.");
        }
    }
}
