package com.tinkerforge;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletVoltageCurrentActions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Get Voltage")
    public @ActionOutput(name = "voltage", type="int")
           Map<String, Object> brickletVoltageCurrentGetVoltage(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletVoltageCurrent)this.handler.getDevice()).getVoltage();
        result.put("voltage", value);
        return result;
    }

    public static Map<String, Object> brickletVoltageCurrentGetVoltage(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletVoltageCurrentActions) {
            return ((BrickletVoltageCurrentActions) actions).brickletVoltageCurrentGetVoltage();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletVoltageCurrentActions class.");
        }
    }

    @RuleAction(label = "Get Current")
    public @ActionOutput(name = "current", type="int")
           Map<String, Object> brickletVoltageCurrentGetCurrent(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletVoltageCurrent)this.handler.getDevice()).getCurrent();
        result.put("current", value);
        return result;
    }

    public static Map<String, Object> brickletVoltageCurrentGetCurrent(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletVoltageCurrentActions) {
            return ((BrickletVoltageCurrentActions) actions).brickletVoltageCurrentGetCurrent();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletVoltageCurrentActions class.");
        }
    }

    @RuleAction(label = "Get Power")
    public @ActionOutput(name = "power", type="int")
           Map<String, Object> brickletVoltageCurrentGetPower(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletVoltageCurrent)this.handler.getDevice()).getPower();
        result.put("power", value);
        return result;
    }

    public static Map<String, Object> brickletVoltageCurrentGetPower(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletVoltageCurrentActions) {
            return ((BrickletVoltageCurrentActions) actions).brickletVoltageCurrentGetPower();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletVoltageCurrentActions class.");
        }
    }

    @RuleAction(label = "Get Configuration")
    public @ActionOutput(name = "averaging", type="short")
           @ActionOutput(name = "voltageConversionTime", type="short")
           @ActionOutput(name = "currentConversionTime", type="short")
           Map<String, Object> brickletVoltageCurrentGetConfiguration(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletVoltageCurrent.Configuration value = ((BrickletVoltageCurrent)this.handler.getDevice()).getConfiguration();
        result.put("averaging", value.averaging);
        result.put("voltageConversionTime", value.voltageConversionTime);
        result.put("currentConversionTime", value.currentConversionTime);
        return result;
    }

    public static Map<String, Object> brickletVoltageCurrentGetConfiguration(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletVoltageCurrentActions) {
            return ((BrickletVoltageCurrentActions) actions).brickletVoltageCurrentGetConfiguration();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletVoltageCurrentActions class.");
        }
    }

    @RuleAction(label = "Get Calibration")
    public @ActionOutput(name = "gainMultiplier", type="int")
           @ActionOutput(name = "gainDivisor", type="int")
           Map<String, Object> brickletVoltageCurrentGetCalibration(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletVoltageCurrent.Calibration value = ((BrickletVoltageCurrent)this.handler.getDevice()).getCalibration();
        result.put("gainMultiplier", value.gainMultiplier);
        result.put("gainDivisor", value.gainDivisor);
        return result;
    }

    public static Map<String, Object> brickletVoltageCurrentGetCalibration(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletVoltageCurrentActions) {
            return ((BrickletVoltageCurrentActions) actions).brickletVoltageCurrentGetCalibration();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletVoltageCurrentActions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="short[]")
           @ActionOutput(name = "firmwareVersion", type="short[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickletVoltageCurrentGetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletVoltageCurrent.Identity value = ((BrickletVoltageCurrent)this.handler.getDevice()).getIdentity();
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletVoltageCurrentGetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletVoltageCurrentActions) {
            return ((BrickletVoltageCurrentActions) actions).brickletVoltageCurrentGetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletVoltageCurrentActions class.");
        }
    }
}
